"""CVAT authentication utilities."""
import os
from getpass import getpass
from typing import Optional

from cvat_sdk import Client


def get_host() -> str:
    """Get the CVAT backend host."""
    cvat_host = os.getenv("CVAT_HOST")
    if cvat_host is None:
        cvat_host = input("CVAT Host: ")
    return cvat_host.rstrip("/")


def get_username() -> str:
    """Get the CVAT user."""
    username = os.getenv("CVAT_USERNAME")
    if username is None:
        username = input("CVAT User: ")
    return username


def get_password() -> str:
    """Get the CVAT password."""
    password = os.getenv("CVAT_PASSWORD")
    if password is None:
        password = getpass()
    return password


def get_org() -> str:
    """Get the CVAT organization name."""
    org = os.getenv("CVAT_ORG")
    if org is None:
        org = input("CVAT Org: ")
    return org


def get_cvat_client(
    username: Optional[str] = None,
    password: Optional[str] = None,
    org: Optional[str] = None,
) -> Client:
    """Get the CVAT SDK high-level client object."""
    if username is None:
        username = get_username()
    if password is None:
        password = get_password()
    if org is None:
        org = get_org()
    client = Client(url=get_host(), check_server_version=False)
    credentials = (username, password)
    client.login(credentials)
    client.api_client.set_default_header("x-organization", org)
    return client



"""Upload images to a CVAT task."""
from pathlib import Path
from typing import Any, Optional, Union

import randomname
from cvat_sdk.core.helpers import TqdmProgressReporter
from cvat_sdk.core.proxies.tasks import ResourceType
from tqdm import tqdm

from .auth import get_cvat_client

VALID_IMAGE_FORMATS = [".jpg", ".jpeg", ".png"]


def create_task(name: str, project_id: int, segment_size: int = 25) -> dict[str, Any]:
    """Create a new task (without attached images/videos)."""
    data = {"name": name, "project_id": project_id, "segment_size": segment_size}
    with get_cvat_client() as client:
        task, _ = client.tasks.api.create(data)
    return task.to_dict()


def upload_image_list(task_id: int, image_list: list[Union[str, Path]]) -> None:
    """Upload a list of images to a task."""
    with get_cvat_client() as client:
        task = client.tasks.retrieve(task_id)
        task.upload_data(
            ResourceType.LOCAL,
            [str(i) for i in image_list],
            pbar=TqdmProgressReporter(tqdm()),
        )


def upload_images(
    image_directory: Union[str, Path],
    project_id: Optional[int] = None,
    task_name: Optional[str] = None,
    segment_size: int = 25,
) -> None:
    """Upload images from a directory to a task."""
    if project_id is None:
        raise ValueError("project_id must be specified")
    if task_name is None:
        task_name = randomname.get_name()
    task = create_task(task_name, project_id, segment_size)
    task_id = task["id"]
    image_directory = Path(image_directory)
    images = []
    for extension in VALID_IMAGE_FORMATS:
        images.extend(image_directory.glob(f"*{extension}"))
    upload_image_list(task_id, sorted(images))
    print(f"Uploaded {len(images)} images to task {task_id}")
